/* NessusJ
 * Copyright (C) 1998 Alexis de Bernis
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This class prepares the connection to nessusd. It retrieves
 * the plugin list.
 *
 *  Please run class NessusJ instead.
 */

import java.lang.*;
import java.util.*;
import java.awt.*;
import java.net.*;
import java.io.*;
import com.sun.java.swing.*;
import com.sun.java.swing.border.*;
import java.awt.event.*;


public class NessusdConnection extends JFrame implements WindowListener,ActionListener
{

    JButton connect,cancel;
    JTextField host,port,login;
    JPasswordField password;
    NessusJ parent;
    DataInputStream fromNessusd;
    DataOutputStream forNessusd;
    Vector plugins;
    Hashtable families;
    JLabel icUser,icComputer;
    JPanel north,center,south,fields1,fields2;
    JPanel computer,user,dlines,buttons,labels1,labels2;
    Preferences prefs;

    NessusdConnection(NessusJ parent, Preferences prefs)
    {
        super("NessusJ");

        addWindowListener(this);

        icUser=new JLabel(new ImageIcon("user.gif"));
        icComputer=new JLabel(new ImageIcon("computer.gif"));

        this.prefs=prefs;
        this.parent=parent;           // In order to control below (fuck multithreading!)
        plugins=new Vector();
        families=new Hashtable();

        host=new JTextField(25);
        port=new JTextField(25);
        login=new JTextField(25);
        password=new JPasswordField(25);

        cancel=new JButton("Cancel");
        cancel.addActionListener(this);
        connect=new JButton("Connect");
        connect.addActionListener(this);

        north=new JPanel(new BorderLayout(0,10));
        center=new JPanel(new BorderLayout());
        south=new JPanel(new BorderLayout());
        dlines=new JPanel(new GridLayout(2,0));
        computer=new JPanel(new GridLayout(0,2));
        user=new JPanel(new GridLayout(0,2));
        buttons=new JPanel(new FlowLayout(FlowLayout.CENTER,50,20));
        labels1=new JPanel(new GridLayout(2,2));
        labels2=new JPanel(new GridLayout(2,2));
        fields1=new JPanel(new GridLayout(2,0));
        fields2=new JPanel(new GridLayout(2,0));

        dlines.add(new HRule(412,1,0,1));
        dlines.add(new HRule(412,1,0,1));

        labels1.add(icComputer);
        labels1.add(new JLabel("nessusd host : ",JLabel.RIGHT));
        labels1.add(new JLabel(" "));
        labels1.add(new JLabel("Port : ",JLabel.RIGHT));

        fields1.add(host);
        fields1.add(port);

        computer.add(labels1);
        computer.add(fields1);
        computer.setBorder(new TitledBorder(new EtchedBorder(),"Nessusd informations"));

        north.add("North",new JLabel("New Session setup",JLabel.CENTER));
        north.add("Center",dlines);
        north.add("South",computer);

        center.add("North",new JLabel(" "));
        center.add("South",new JLabel(" "));

        labels2.add(icUser);
        labels2.add(new JLabel("Login : ",JLabel.RIGHT));
        labels2.add(new JLabel(" "));
        labels2.add(new JLabel("Password : ",JLabel.RIGHT));

        fields2.add(login);
        fields2.add(password);

        user.add(labels2);
        user.add(fields2);
        user.setBorder(new TitledBorder(new EtchedBorder(),"User informations"));

        buttons.add(cancel);
        buttons.add(connect);

        south.add("North",user);
        south.add("Center",buttons);

        getContentPane().add("North",north);
        getContentPane().add("Center",center);
        getContentPane().add("South",south);


        password.setEchoChar('*');
        port.setText("3000");

        setSize(412,290);
        setResizable(false);
        setVisible(true);
    }

    private boolean makePlugInTable()
    {
        String received=new String("");
        int id;
        String name,copyright,description,summary,type,family;

//*        System.out.println("Making plug-in table...");

        for(;;)
        {

            try
            {
                received=fromNessusd.readLine();
            }
            catch(IOException e)
            {
                return(false);
            }

//*            System.out.println(received);
            if(received.equals("<|> SERVER"))
                break;

            id=Integer.parseInt(received.substring(0,received.indexOf(" <|> ")));
//*            System.out.println("New Plug-in: "+id);
            received=received.substring(received.indexOf("<|> ")+4);
            name=received.substring(0,received.indexOf(" <|> "));
            received=received.substring(received.indexOf("<|> ")+4);
            type=received.substring(0,received.indexOf(" <|> "));
            received=received.substring(received.indexOf("<|> ")+4);
            copyright=received.substring(0,received.indexOf(" <|> "));
            received=received.substring(received.indexOf("<|> ")+4);
            description=received.substring(0,received.indexOf(" <|> "));
            received=received.substring(received.indexOf("<|> ")+4);
            summary=received.substring(0,received.indexOf(" <|> "));
            description=description.replace(';','\n');
            received=received.substring(received.indexOf("<|> ")+4);
            family=received;

//*            System.out.println("Name :        "+name);
//*            System.out.println("Copyright :   "+copyright);
//*            System.out.println("Description : "+description);
//*            System.out.println("Summary :     "+summary);
//*            System.out.println("Type    :     "+type);
//*            System.out.println("Family  :     "+family+"\n");

            PlugIn temp=new PlugIn(id,
                                   name,
                                   copyright,
                                   description,
                                   summary,
                                   type,
                                   family,
                                   prefs.isPluginActivated(name),
                                   prefs);

            plugins.addElement(temp);
            registerFamily(temp,family);

        }
            
//*        System.out.println("Finished to make plug-in table");
        return(true);

    }



    private boolean connectNessusd(String host,int port,String login,String password)
    {
        String received;

//*        System.out.println("Connecting Nessusd:");
//*        System.out.println("Host:   "+host);
//*        System.out.println("Port:   "+port);
//*        System.out.println("login:  "+login);
//*        System.out.println("passwd: "+password+"\n");


        try
        {
            Socket connection=new Socket(host,port);
            fromNessusd=new DataInputStream(new BufferedInputStream(connection.getInputStream()));
            forNessusd=new DataOutputStream(connection.getOutputStream());
        }
        catch(Exception e)
        {
            JOptionPane.showMessageDialog(this,"Error: Can't connect, bad host or port closed","Error",JOptionPane.ERROR_MESSAGE);
            return(false);
        }

//*        System.out.println("Connection OK!");
	
	try
        {
            forNessusd.writeBytes("< NTP/1.0 >\n");
            received=fromNessusd.readLine();
//*        System.out.println("Received : "+received);

                if(!received.equals("< NTP/1.0 >"))
                {
                    JOptionPane.showMessageDialog(this,"Error: unexpected reply received: "+received,"Error",JOptionPane.ERROR_MESSAGE);
                    return(false);
                }
            
            forNessusd.writeBytes(login+"\n");
//*            System.out.println("Send : "+login);
            forNessusd.writeBytes(password+"\n");
//*            System.out.println("Send : "+password);

            received=fromNessusd.readLine();
            if(!received.equals("User : Password : SERVER <|> PLUGIN_LIST <|>"))
            {
                JOptionPane.showMessageDialog(this,"Error: unexpected response: "+received+"\nMay be a bad login or password","Error",JOptionPane.ERROR_MESSAGE);
                return(false);
            }

            return(makePlugInTable());
            
        }catch(IOException e)
        {
            JOptionPane.showMessageDialog(this,"Error: Connection closed by foreign host","Error",JOptionPane.ERROR_MESSAGE);
            return(false);
        }
    }


    public void back()
    {
        dispose();
        parent.show();
    }

    public void registerFamily(PlugIn plugin, String family)
    {
        Family temp;

        temp=(Family) families.get(family);

        if(temp==null)
        {
            temp=new Family(family);
            families.put(family,temp);
        }

        temp.addChild(plugin);
    }


    public void windowClosing(WindowEvent evt)
    {
        dispose();
        parent.show();
    }

    public void actionPerformed(ActionEvent evt)
    {
        String command=evt.getActionCommand();

        if(command.equals("Connect"))
        {

            if(!connectNessusd(host.getText(),
                              Integer.parseInt(port.getText()),
                              login.getText(),
                              password.getText()))
            {
                JOptionPane.showMessageDialog(this,"Error: an error ocurred during connection","Error",JOptionPane.ERROR_MESSAGE);
            }

            dispose();
            new NessusdLauncher(plugins,
                                fromNessusd,
                                forNessusd,
                                host.getText(),
                                this,
                                prefs,
                                families);
        }
        
        if(command.equals("Cancel"))
        {
            dispose();
            parent.show();
        }


    }
    


    public void windowClosed(WindowEvent evt ) { }
    public void windowOpened(WindowEvent evt) { }
    public void windowOpening(WindowEvent evt ) { }
    public void windowIconified(WindowEvent evt ) { }
    public void windowActivated(WindowEvent evt ) { }
    public void windowDeiconified(WindowEvent evt ) { }
    public void windowDeactivated(WindowEvent evt ) { }



}
