/* NessusJ
 * Copyright (C) 1998 Alexis de Bernis
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This class display the report at the end of the attack.
 *
 *  Please run class NessusJ instead.
 */


import java.awt.*;
import java.util.Vector;
import java.io.*;
import java.awt.event.*;
import com.sun.java.swing.*;
import com.sun.java.swing.border.*;


class NessusdReport extends JFrame implements WindowListener,ActionListener
{
    JButton close,save,saveHTML;
    Box host,port;
    JScrollPane hostsPane,portsPane;
    JTextArea description;
    JPanel button,lists;
    Vector hosts;
    JFrame parent;
    String curHost=" ";
    int curPort=0;

    NessusdReport(JFrame parent, Vector hosts)
    {
        super("Report");

        this.hosts=hosts;
        this.parent=parent;

        addWindowListener(this);
        close=new JButton("Close");
        close.addActionListener(this);
        save=new JButton("Save");
        save.addActionListener(this);
        saveHTML=new JButton("Save to HTML");
        saveHTML.addActionListener(this);

        host=new Box(BoxLayout.Y_AXIS);
        port=new Box(BoxLayout.Y_AXIS);
        hostsPane=new JScrollPane(host);
        hostsPane.setBorder(new TitledBorder(new EtchedBorder(),"Hosts"));
        portsPane=new JScrollPane(port);
        portsPane.setBorder(new TitledBorder(new EtchedBorder(),"Ports"));

        description=new JTextArea("",10,40);
        description.setEditable(false);
        description.setLineWrap(true);
        description.setWrapStyleWord(true);


        button=new JPanel(new FlowLayout());
        button.add(close);
        button.add(save);
        button.add(saveHTML);

        lists=new JPanel(new GridLayout(0,2));
        lists.add(hostsPane);
        lists.add(portsPane);

        getContentPane().setLayout(new BorderLayout());
        getContentPane().add("North",lists);
        getContentPane().add("Center",description);
        getContentPane().add("South",button);

        refreshHosts();

        pack();
        setResizable(false);
        setVisible(true);
    }

    public void refreshHosts()
    {
        host.removeAll();

        for(int i=0;i<hosts.size();i++)
        {
            String icon="l_green.gif";
            Host current=(Host) hosts.elementAt(i);

            if(current.getInfos().size()!=0)
                icon="l_orange.gif";

            if(current.getHoles().size()!=0)
                icon="l_red.gif";

            JButton toAdd=new JButton(current.getName(),new ImageIcon(icon));
            toAdd.setBorder(new EmptyBorder(0,0,0,0));
            toAdd.addActionListener(this);

            host.add(toAdd);
        }

        validateTree();
    }



    public void refreshDescription(int port2show)
    {
        curPort=port2show;
        Host current=getHost(curHost);
        Vector holes=current.getHoles();
        Vector infos=current.getInfos();

        clearText(description);

        description.append("Holes :\n");

        for(int i=0;i<holes.size();i++)
        {
            if(((Hole) holes.elementAt(i)).getPort()==port2show)
                description.append(((Hole) holes.elementAt(i)).getDescription()+"\n");
        }

        description.append("Infos :\n");     //Separate infos and holes

        for(int i=0;i<infos.size();i++)
        {
            if(((Info) infos.elementAt(i)).getPort()==port2show)
                description.append(((Info) infos.elementAt(i)).getDescription()+"\n");
        }
    }
        



    public void refreshPort()
    {
        String toAddIcon,toAddText;

        port.removeAll();

        Host current=getHost(curHost);

        Vector ports=current.getOpenPorts();

        for(int i=0;i<ports.size();i++)
        {
            int actual=((Integer) ports.elementAt(i)).intValue();

            toAddIcon="l_green.gif";
            toAddText=String.valueOf(actual);

            if(actual==-1)
                toAddText="General";    


            if(isInfo(current,actual))
            {
                toAddIcon="l_orange.gif";
                if(actual==-1)
                    toAddText="General";
            }

            if(isHole(current,actual))
            {
                toAddIcon="l_red.gif";
                if(actual==-1)
                    toAddText="General";
            }

            JButton toAddButton=new JButton(toAddText,new ImageIcon(toAddIcon));
            toAddButton.setBorder(new EmptyBorder(0,0,0,0));
            toAddButton.addActionListener(this);
            port.add(toAddButton);

        }

//        validateTree();
        pack();

    }

    private boolean isHole(Host host,int toTest)
    {
        
        Vector holes=host.getHoles();

        for(int i=0;i<holes.size();i++)
        {
            if(((Hole) holes.elementAt(i)).getPort()==toTest)
                return(true);
        }
        return(false);
    }

    private boolean isInfo(Host host,int toTest)
    {
        
        Vector infos=host.getInfos();

        for(int i=0;i<infos.size();i++)
        {
            if(((Info) infos.elementAt(i)).getPort()==toTest)
                return(true);
        }
        return(false);
    }

    private Host getHost(String name)
    {
        for(int i=0;i<hosts.size();i++)
        {
            if(((Host) hosts.elementAt(i)).getName().equals(name))
                return((Host) hosts.elementAt(i));
        }

        return(null);
    }

    private void clearText(JTextArea area)
    {
        area.setText("");
    }

    private void saveReport()
    {
        FileDialog dialog=new FileDialog(this,"Save Report as...",FileDialog.SAVE);
        String filename;
        DataOutputStream toFile;

        dialog.setFile("*.nsr");
        dialog.show();

        if((filename=dialog.getFile())==null)
            return;

        try
        {
            toFile=new DataOutputStream(new FileOutputStream(dialog.getDirectory()+filename));
        }catch(IOException e)
        {
            System.out.println("Error: impossible to create file : "+filename);
            return;
        }

        for(int i=0;i<hosts.size();i++)
        {
            String name=((Host) hosts.elementAt(i)).getName();
            Vector infos=((Host) hosts.elementAt(i)).getInfos();
            Vector holes=((Host) hosts.elementAt(i)).getHoles();

            for(int j=0;j<infos.size();j++)
            {
                String port=new Integer(((Info) infos.elementAt(j)).getPort()).toString();

                if(port.equals("-1"))
                    port="Generic";
                String toWrite=name+" "+
                               port+" "+
                               "INFO"+" "+
                               ((Info) infos.elementAt(j)).getDescription();
                toWrite=toWrite.replace('\n',';')+"\n";

                port=null;

                try
                {
                    toFile.writeBytes(toWrite);
                }catch(IOException e)
                {
                    System.out.println("Error : impossible to write report");
                    return;
                }

                toWrite=null;
            }

            for(int j=0;j<holes.size();j++)
            {
                String port=new Integer(((Hole) holes.elementAt(j)).getPort()).toString();

                if(port.equals("-1"))
                    port="Generic";

                String toWrite=name+" "+
                               port+" "+
                               "REPORT"+" "+ 
                               ((Hole) holes.elementAt(j)).getDescription();
                toWrite=toWrite.replace('\n',';')+"\n";
                try
                {
                    toFile.writeBytes(toWrite);
                }catch(IOException e)
                {
                    System.out.println("Error : impossible to write report");
                    return;
                }

                toWrite=null;
            }

            name=null;
            infos=null;
            holes=null;
        }

    }

    public void back()
    {
        dispose();
        if(parent instanceof NessusdStatus)
        {
            ((NessusdStatus) parent).back();
        }else
        {
            parent.show();
        }
    }

    private void saveReportToHTML()
    {

        FileDialog dialog=new FileDialog(this,"Save Report as...",FileDialog.SAVE);
        String filename;
        DataOutputStream toFile;

        dialog.setFile("*.html");
        dialog.show();

        if((filename=dialog.getFile())==null)
            return;

        try
        {
            toFile=new DataOutputStream(new FileOutputStream(dialog.getDirectory()+filename));
        }catch(IOException e)
        {
            System.out.println("Error: impossible to create file : "+filename);
            return;
        }

        try
        {
            toFile.writeBytes("<HTML>\n<HEAD>\n<TITLE>Nessus Report</TITLE>\n</HEAD>\n");
            toFile.writeBytes("<BODY BGCOLOR=\"#FFFFFF\" TEXT=\"#000000\">\n");
            toFile.writeBytes("<CENTER><U><H1>NessusJ Report</H1></U></CENTER>\n");
            toFile.writeBytes("<BR>\n<BR>\n");

            for(int i=0;i<hosts.size();i++)
            {

                toFile.writeBytes("<H2><B>Host : "+((Host) hosts.elementAt(i)).getName()+"</B></H2>\n");
                toFile.writeBytes("<BR>\n<UL>\n");
                toFile.writeBytes("<B><LI>Holes : </B>\n");
                toFile.writeBytes("<UL>\n");
            
                Vector holes=((Host) hosts.elementAt(i)).getHoles();

                for(int j=0;j<holes.size();j++)
                {
                    int p=((Hole) holes.elementAt(j)).getPort();
                    String portNumber;

                    if(p==-1)
                        portNumber="General";
                    else
                        portNumber=String.valueOf(p);


                    toFile.writeBytes("<LI>"+portNumber+" : "+((Hole) holes.elementAt(j)).getDescription()+"\n");
                }

                toFile.writeBytes("</UL>\n<B><LI>Infos :</B>\n<UL>\n");

                Vector infos=((Host) hosts.elementAt(i)).getInfos();

                for(int j=0;j<infos.size();j++)
                {
                    int p=((Info) infos.elementAt(j)).getPort();
                    String portNumber;

                    if(p==-1)
                        portNumber="General";
                    else
                        portNumber=String.valueOf(p);

                    toFile.writeBytes("<LI>"+portNumber+" : "+((Info) infos.elementAt(j)).getDescription()+"\n");
                }

                toFile.writeBytes("</UL>\n</UL>\n<HR>\n");

            }

                toFile.writeBytes("<I>This file was automagically created by <A HREF=\"http://www.nessus.org/NessusJ\">NessusJ</A>.</I>\n");
                toFile.writeBytes("</BODY>\n</HTML>");
        } catch(IOException e)
        {
            System.out.println("Error writing report to HTML : "+e);
            return;
        }
    }

    public void windowClosing(WindowEvent evt)
    {
        back();
    }


    public void windowClosed(WindowEvent evt ) { }
    public void windowOpened(WindowEvent evt) { }
    public void windowOpening(WindowEvent evt ) { }
    public void windowIconified(WindowEvent evt ) { }
    public void windowActivated(WindowEvent evt ) { }
    public void windowDeiconified(WindowEvent evt ) { }
    public void windowDeactivated(WindowEvent evt ) { }


    public void actionPerformed(ActionEvent evt)
    {
        String command=evt.getActionCommand();

        if(command.equals("Close"))
        {
            back();
            return;
        }

        if(command.equals("Save"))
        {
            saveReport();
            return;
        }

        if(command.equals("Save to HTML"))
        {
            saveReportToHTML();
            return;
        }
        
        Host temp;

        if((temp=getHost(command))!=null)
        {
            curHost=command;
            refreshPort();
            return;

        } else
        {
            int portToRefresh;

            if(command.equals("General"))
                portToRefresh=-1;
            else
                portToRefresh=Integer.parseInt(command);

            refreshDescription(portToRefresh);

        }

    }

    public String getCurrentHost()
    {
        return(curHost);
    }

    public int getCurrentPort()
    {
        return(curPort);
    }

}
