/* NessusJ
 * Copyright (C) 1998 Alexis de Bernis
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * This class display the status window.
 *
 *  Please run class NessusJ instead.
 */


import java.lang.*;
import java.io.*;
import java.awt.*;
import java.util.*;
import com.sun.java.swing.*;
import java.awt.event.*;



class NessusdStatus extends JFrame implements ActionListener
{
    DataInputStream fromNessusd;
    DataOutputStream forNessusd;
    JScrollPane pane;
    JPanel infos;
    Box list;
    JButton stopAll;
    JPanel button,main;
    Vector hosts;
    NessusdListener listener;
    NessusdLauncher parent;
    NessusdReport report;
    JLabel curHost,numHoles,numInfos,curPlugin;
    String pluginsOrder[];

    NessusdStatus(NessusdLauncher parent,
                  DataInputStream fromNessusd,
                  DataOutputStream forNessusd,
                  String machine,
                  String pluginsOrder[])
    {
        super("Attack status");

        this.parent=parent;
        this.fromNessusd=fromNessusd;
        this.forNessusd=forNessusd;
        this.pluginsOrder=pluginsOrder;

        getContentPane().setLayout(new BorderLayout());

//        pane.getViewPort().add(list);
        list=new Box(BoxLayout.Y_AXIS);
        pane=new JScrollPane(list);

        stopAll=new JButton("Stop All!");

        button=new JPanel(new FlowLayout());
        button.add(stopAll);

        curHost=new JLabel("No host selected");
        numHoles=new JLabel("---");
        numInfos=new JLabel("---");
        curPlugin=new JLabel("---");

        infos=new JPanel(new GridLayout(0,2));
        infos.add(new JLabel("Host :"));
        infos.add(curHost);
        infos.add(new JLabel("Holes already founds :"));
        infos.add(numHoles);
        infos.add(new JLabel("Infos already founds :"));
        infos.add(numInfos);
        infos.add(new JLabel("Plugin being tested :"));
        infos.add(curPlugin);

        main=new JPanel(new BorderLayout());
        main.add("Center",pane);
        main.add("South",infos);

        getContentPane().add("Center",main);
        getContentPane().add("South",button);

        hosts=new Vector();
        Host temp=new Host(machine);
        hosts.addElement(temp);
        list.add(temp);
//        list.add(Box.createGlue());

        pack();
        Dimension size=getSize();
        setSize(new Dimension(size.width,450));
        setVisible(true);
        report=new NessusdReport(this,hosts);
        listener=new NessusdListener(this, fromNessusd, forNessusd);
        listener.start();
    }

    public void jobFinished()
    {
        dispose();
//        new NessusdReport(this,hosts);
    }

    public void postPort(String victim,int port)
    {
        Host host=getHost(victim);
        host.addOpenPort(new Integer(port));

        if(report.getCurrentHost().equals(victim))
            report.refreshPort();
    }

    public void refreshHost(String victim,String cur)
    {
        Host host=getHost(victim);
        host.setInfo(cur);

        if((!host.isPortScanning()) && host.getName().equals(curHost.getText()))
            curPlugin.setText(pluginsOrder[Integer.parseInt(cur.substring(0,cur.indexOf("/")))]);
    }

    public void postHole(String victim, Hole hole)
    {
        Host host=getHost(victim);
        host.addHole(hole);
        markAsOpen(new Integer(hole.getPort()),host);
        if(host.getName().equals(curHost.getText()))
            numHoles.setText(String.valueOf(host.getHoleCount()));

        report.refreshHosts();

        if(report.getCurrentHost().equals(victim))
            report.refreshPort();

        if(report.getCurrentHost().equals(victim) &&
           report.getCurrentPort()==hole.getPort())
           report.refreshDescription(hole.getPort());

    }

    public void postInfo(String victim, Info info)
    {
        Host host=getHost(victim);
        host.addInfo(info);
        markAsOpen(new Integer(info.getPort()),host);
        if(host.getName().equals(curHost.getText()))
            numInfos.setText(String.valueOf(host.getInfoCount()));

        report.refreshHosts();

        if(report.getCurrentHost().equals(victim))
            report.refreshPort();

        if(report.getCurrentHost().equals(victim) &&
           report.getCurrentPort()==info.getPort())
           report.refreshDescription(info.getPort());
    }


    public Host getHost(String host)
    {
        for(int i=0;i<hosts.size();i++)
        {
            if(((Host) hosts.elementAt(i)).getName().equals(host))
                return((Host) hosts.elementAt(i));
        }

//*        System.out.println("Unknown machine : "+host+". Creating...");
        Host temp=new Host(host);
        hosts.addElement(temp);
        list.add(temp);
        pack();

        report.refreshHosts();

        return(temp);
    }

    private void markAsOpen(Integer port,Host host)
    {
        Vector openPorts=host.getOpenPorts();

        for(int i=0;i<openPorts.size();i++)
        {
            if(((Integer) openPorts.elementAt(i)).equals(port))
                return;
        }

        host.addOpenPort(port);
    }


    public void stopAttack(Host host)
    {
        String separator=" <|> ";
        String entity="CLIENT";

        String message=entity+separator+
                       "STOP_ATTACK"+separator+
                       host.getName()+separator+
                       entity+"\n";

        try
        {
            forNessusd.writeBytes(message);
        }catch(IOException e)
        {
            System.out.println("Error: impossible to join Nessusd");
        }

    }

    private void stopAllAttacks()
    {
        for(int i=0;i<hosts.size();i++)
            stopAttack((Host) hosts.elementAt(i));
    }



    public void back()
    {
        dispose();
        parent.back();
    }




    public void actionPerformed(ActionEvent evt)
    {
        String command=evt.getActionCommand();

        if(command.equals("Stop All!"))
            stopAllAttacks();
    }

    public void hostSelected(Host source)
    {
        curHost.setText(source.getName());
        numHoles.setText(String.valueOf(source.getHoleCount()));
        numInfos.setText(String.valueOf(source.getInfoCount()));
    }

}
